@@ -30,13 +30,13 @@
 # THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 
-""" Identify specific nodes in a JSON document (according to draft 05) """
+""" Identify specific nodes in a JSON document (according to draft 08) """
 
-# http://tools.ietf.org/html/draft-ietf-appsawg-json-pointer-05
+# http://tools.ietf.org/html/draft-ietf-appsawg-json-pointer-08
 
 # Will be parsed by setup.py to determine package metadata
 __author__ = 'Stefan Kögl <stefan@skoegl.net>'
-__version__ = '0.5'
+__version__ = '0.7'
 __website__ = 'https://github.com/stefankoegl/python-json-pointer'
 __license__ = 'Modified BSD License'
 
@@ -49,6 +49,11 @@ except ImportError: # Python 3
     izip = zip
 
 from itertools import tee
+import re
+
+
+# array indices must not contain leading zeros, signs, spaces, decimals, etc
+RE_ARRAY_INDEX=re.compile('0|[1-9][0-9]*$')
 
 
 class JsonPointerException(Exception):
@@ -100,38 +105,6 @@ def resolve_pointer(doc, pointer, default=_nothing):
     return pointer.resolve(doc, default)
 
 
-def set_pointer(doc, pointer, value):
-    """
-    Set a field to a given value
-
-    The field is indicates by a base location that is given in the constructor,
-    and an optional relative location in the call to set. If the path doesn't
-    exist, it is created if possible
-
-    >>> obj = {"foo": 2}
-    >>> pointer = JsonPointer('/bar')
-    >>> pointer.set(obj, 'one', '0')
-    >>> pointer.set(obj, 'two', '1')
-    >>> obj == {'foo': 2, 'bar': ['one', 'two']}
-    True
-
-    >>> obj = {"foo": 2, "bar": []}
-    >>> pointer = JsonPointer('/bar')
-    >>> pointer.set(obj, 5, '0/x')
-    >>> obj == {'foo': 2, 'bar': [{'x': 5}]}
-    True
-
-    >>> obj = {'foo': 2, 'bar': [{'x': 5}]}
-    >>> pointer = JsonPointer('/bar/0')
-    >>> pointer.set(obj, 10, 'y/0')
-    >>> obj == {'foo': 2, 'bar': [{'y': [10], 'x': 5}]}
-    True
-    """
-
-    pointer = JsonPointer(pointer)
-    pointer.set(doc, value)
-
-
 class JsonPointer(object):
     """ A JSON Pointer that can reference parts of an JSON document """
 
@@ -149,6 +122,9 @@ class JsonPointer(object):
     def to_last(self, doc, default=_nothing):
         """ Resolves ptr until the last step, returns (sub-doc, last-step) """
 
+        if not self.parts:
+            return doc, None
+
         for part in self.parts[:-1]:
             doc = self.walk(doc, part)
 
@@ -174,48 +150,6 @@ class JsonPointer(object):
     get = resolve
 
 
-    def set(self, doc, value, path=None):
-        """ Sets a field of doc to value
-
-        The location of the field is given by the pointers base location and
-        the optional path which is relative to the base location """
-
-        fullpath = list(self.parts)
-
-        if path:
-            fullpath += path.split('/')
-
-
-        for part, nextpart in pairwise(fullpath):
-            try:
-                doc = self.walk(doc, part)
-            except JsonPointerException:
-                step_val = [] if nextpart.isdigit() else {}
-                doc = self._set_value(doc, part, step_val)
-
-        self._set_value(doc, fullpath[-1], value)
-
-
-    @staticmethod
-    def _set_value(doc, part, value):
-        part = int(part) if part.isdigit() else part
-
-        if isinstance(doc, dict):
-            doc[part] = value
-
-        if isinstance(doc, list):
-            if len(doc) < part:
-                doc[part] = value
-
-            if len(doc) == part:
-                doc.append(value)
-
-            else:
-                raise IndexError
-
-        return doc[part]
-
-
     def get_part(self, doc, part):
         """ Returns the next step in the correct type """
 
@@ -227,11 +161,11 @@ class JsonPointer(object):
             if part == '-':
                 return part
 
-            try:
-                return int(part)
-            except ValueError:
+            if not RE_ARRAY_INDEX.match(str(part)):
                 raise JsonPointerException("'%s' is not a valid list index" % (part, ))
 
+            return int(part)
+
         else:
             raise JsonPointerException("Unknown document type '%s'" % (doc.__class__,))
 
@@ -271,6 +205,23 @@ class JsonPointer(object):
              self.parts[:len(ptr.parts)] == ptr.parts
 
 
+    def __eq__(self, other):
+        """ compares a pointer to another object
+
+        Pointers can be compared by comparing their strings (or splitted
+        strings), because no two different parts can point to the same
+        structure in an object (eg no different number representations) """
+
+        if not isinstance(other, JsonPointer):
+            return False
+
+        return self.parts == other.parts
+
+
+    def __hash__(self):
+        return hash(tuple(self.parts))
+
+
 def pairwise(iterable):
     "s -> (s0,s1), (s1,s2), (s2, s3), ..."
     a, b = tee(iterable)
